function CAMS = read_cams_radiation_csv(csv_filename, geopoint, ydoy_begin, ydoy_end, duration, time, verbose_output)
% function CAMS = read_cams_radiation_csv(csv_filename, geopoint, ydoy_begin, ydoy_end, duration, time, verbose_output)
% Read a CAMS Radiation time series.
% csv_filename : CSV file to read
% geopoint : [lat lon h]
% ydoy_begin : [yyyy doy]
% ydoy_end : [yyyy doy]
% duration : 'd', 'h', 15 or 1
% time : UT or TST
% verbose_output : 1 if verbose CSV file format (supported if 1-minute UT time series only)
%
% Example:
% read_cams_radiation_csv('camsFileExample.csv', [0.0 0.0 0], [2005 19], [2005 20], 'd', 'UT', 0)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ymd_begin = yd_to_ymd(ydoy_begin);
ymd_end   = yd_to_ymd(ydoy_end);

% verbose CSV file format is supported if 1-minute UT time series only
verbose_output=0;
format='csv';
if verbose_output
    if duration == 1 && strcmp(time, 'UT')
        format='csv_verbose';
    else
        error('Incorrect verbose_output parameter');
    end
end

switch(duration)
    case 1 % min
        intraday = 1;
        dt = duration/60; % duration in hours
        wps_duration='PT01M';
    case 15 % min
        intraday = 1;
        dt = duration/60;
        wps_duration='PT15M';
    case {'h', 60}
        intraday = 1;
        duration = 60;
        dt = 1;
        wps_duration='PT01H';
    case 'd'
        intraday = 0;
        dt = 24;
        wps_duration='P01D';
    otherwise
        error('Incorrect time step parameter');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read output file in columns C.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Format: columns:
% 1. Observation period (ISO 8601)
% 2. TOA. Irradiation on horizontal plane at the top of atmosphere (Wh/m2)
% 3. Clear sky GHI. Clear sky global irradiation on horizontal plane at ground level (Wh/m2)
% 4. Clear sky BHI. Clear sky beam irradiation on horizontal plane at ground level (Wh/m2)
% 5. Clear sky DHI. Clear sky diffuse irradiation on horizontal plane at ground level (Wh/m2)
% 6. Clear sky BNI. Clear sky beam irradiation on mobile plane following the sun at normal incidence (Wh/m2)
% 7. GHI. Global irradiation on horizontal plane at ground level (Wh/m2)
% 8. BHI. Beam irradiation on horizontal plane at ground level (Wh/m2)
% 9. DHI. Diffuse irradiation on horizontal plane at ground level (Wh/m2)
%10. BNI. Beam irradiation on mobile plane following the sun at normal incidence (Wh/m2)
%11. Reliability. Proportion of reliable data in the summarization (0-1)
%12. sza. Solar zenith angle for the middle of the summarization (deg)
%13. atm. Atmospheric profile code: afglus=U.S. standard afglt=tropical afglms=midlatitude summer afglmw=midlatitude winter afglss=subarctic summer afglsw=subarctic winter
%14. tco3. Total column content of ozone (Dobson unit)
%15. tcwv. Total column content of water vapour (kg/m2)
%16. AOD BC. Partial aerosol optical depth at 550 nm for black carbon
%17. AOD DU. Partial aerosol optical depth at 550 nm for dust
%18. AOD SS. Partial aerosol optical depth at 550 nm for sea salt
%19. AOD OR. Partial aerosol optical depth at 550 nm for organic matter
%20. AOD SU. Partial aerosol optical depth at 550 nm for sulphate
%21. AOD 550. Aerosol optical depth at 550 nm
%22. AOD 1240. Aerosol optical depth at 1240 nm
%23. alpha. Angstroem coefficient for aerosol
%24. Aerosol type. Type of aerosol: -1=no value 5=urban 7=continental clean 8=continental polluted 9=continental average 10=maritime clean 11= maritime polluted 12=maritime tropical 13=antarctic 14=desert
%25. fiso. MODIS-like BRDF parameter fiso
%26. fvol. MODIS-like BRDF parameter fvol
%27. fgeo. MODIS-like BRDF parameter fgeo
%28. albedo. Ground albedo
%29. Cloud optical depth  (value of the nearest acquisition time of the pixel)
%30. Cloud coverage of the pixel (percentage from 0 to 100, value of the nearest acquisition time of the pixel)
%31. Cloud type (value of the nearest acquisition time of the pixel) -1=no value 0=no clouds 5=low-level cloud 6=medium-level cloud 7=high-level cloud 8=thin cloud

% Columns 12 to 31 are available if 1-minute UT time serie with -v option.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fp = fopen(csv_filename,'rt');
if (verbose_output) % if columns 12 to 31 are available
    C = textscan(fp,'%s %f %f %f %f %f %f %f %f %f %f %f %s %f %f %f %f %f %f %f %f %f %f %s %f %f %f %f %f %f %f\n', 'CommentStyle','#', 'Delimiter',';');
else
    C = textscan(fp,'%s %f %f %f %f %f %f %f %f %f %f\n', 'CommentStyle','#', 'Delimiter',';');
end
fclose(fp);

nbval = length(C{1});

%%%%%%%%%%%%%%%%%%%%%%%
% fill CAMS structure
%%%%%%%%%%%%%%%%%%%%%%%

jd0 = ymd_to_jd([ymd_begin(1:3) 0]); % first day at 00:00 (julian day)
jd1 = ymd_to_jd([ymd_end(1:3) 0]); % last day at 00:00 (julian day)
vjd_h0 = jd0:jd1; % list of julian days
ymd = jd_to_ymd(vjd_h0); % list of ymd days
ymd(:, 4) = []; % remove last column

CAMS.name = 'CAMS Radiation';
CAMS.geopoint = geopoint;
CAMS.time = time;

K = 1; % scale factor to apply to irradiation
if (intraday == 1)
    K = dt;
    t = [dt:dt:24]; % list of times for 1 day (end of period)
    nt = length(t); % #data per day
else
    switch(duration)
        case 'd'
            t = 24; % list of times for 1 day (end of period)
            nt = length(t); % #data per day
        otherwise
            error('Incorrect time step parameter');
    end
end

% decode Observation period = Start date/End date
datenums=zeros(nbval, 1); % list of ymdhms times
for i=1:nbval
    D=textscan(C{1}{i}, '%*s %s', 'delimiter', '/');
    datenums(i)=datenum(D{1}, 'yyyy-mm-ddTHH:MM:SS');
end
ymdhms=datevec(datenums); % list of ymdhms times

C2 = C{1,2}/K; % TOA
C3 = C{1,3}/K; % CLS_GHI
C4 = C{1,4}/K; % CLS_BHI
C5 = C{1,5}/K; % CLS_DHI
C6 = C{1,6}/K; % CLS_BNI
C7 = C{1,7}/K; % GHI
C8 = C{1,8}/K; % BHI
C9 = C{1,9}/K; % DHI
C10= C{1,10}/K;% BNI

if (verbose_output) % if columns 12 to 31 are available
    C12 = C{1,12}; % sza
    C23 = C{1,23}; % alpha
end

CAMS.ymd = ymd; % list of ymd days
CAMS.t = t; % list of times for 1 day (end of period)
CAMS.nt = nt; % #data per day
CAMS.ymdhms = reshape(ymdhms,[nt*length(vjd_h0) 6]); % list of ymdhms times

CAMS.TOA     = reshape(C2, [nt length(vjd_h0)]);
CAMS.CLS_GHI = reshape(C3, [nt length(vjd_h0)]);
CAMS.CLS_BHI = reshape(C4, [nt length(vjd_h0)]);
CAMS.CLS_DHI = reshape(C5, [nt length(vjd_h0)]);
CAMS.CLS_BNI = reshape(C6, [nt length(vjd_h0)]);
CAMS.GHI     = reshape(C7,[nt length(vjd_h0)]);
CAMS.BHI     = reshape(C8,[nt length(vjd_h0)]);
CAMS.DHI     = reshape(C9,[nt length(vjd_h0)]);
CAMS.BNI     = reshape(C10,[nt length(vjd_h0)]);

if (verbose_output) % if columns 12 to 31 are available
    CAMS.sza   = reshape(C12, [nt length(vjd_h0)]);
    CAMS.sza(CAMS.sza == 0) = nan;
    CAMS.alpha = reshape(C23, [nt length(vjd_h0)]);
end

